module LAB10_2_TOP(
  // SSD
  output reg [7:0] cathode,
  output reg [3:0] anode,

  // speaker
  output audio_mclk, // master clock
  output audio_lrck, // left-right clock
  output audio_sck,  // serial clock
  output audio_sdin, // serial audio data input
  
  // keyboard
  inout PS2_DATA,
  inout PS2_CLK,
  
  input  clk,        // clock from the crystal
  input  rst_n       // active low reset
);
  
  ///////////////////
  // read keyboard //
  ///////////////////
  
  wire [511:0] key_down;
  wire [8:0]   last_change;
  wire         key_valid;
  
  KeyboardDecoder KeyboardDecoder(
    .key_down,
    .last_change,
    .key_valid,
    .PS2_DATA,
    .PS2_CLK,
    .rst(~rst_n),
    .clk
  );
  
  ////////////////////////////
  // look-up note_frequenct //
  ////////////////////////////
  
  reg [26:0] note_frequency;
  
  always @* begin
    case ('b1)
      key_down[8'h1C]: note_frequency <= 27'd261; // Do
      key_down[8'h1B]: note_frequency <= 27'd293; // Re
      key_down[8'h23]: note_frequency <= 27'd330; // Mi
      key_down[8'h2B]: note_frequency <= 27'd349; // Fa
      key_down[8'h34]: note_frequency <= 27'd392; // So
      key_down[8'h33]: note_frequency <= 27'd440; // La
      key_down[8'h3B]: note_frequency <= 27'd494; // Si
      key_down[8'h15]: note_frequency <= 27'd524; // Do
      key_down[8'h1D]: note_frequency <= 27'd588; // Re
      key_down[8'h24]: note_frequency <= 27'd660; // Mi
      key_down[8'h2D]: note_frequency <= 27'd698; // Fa
      key_down[8'h2C]: note_frequency <= 27'd784; // So
      key_down[8'h35]: note_frequency <= 27'd880; // La
      key_down[8'h3C]: note_frequency <= 27'd988; // Si
      default: note_frequency <= 27'd100_000_000;
    endcase
  end
  
  //////////////////////
  // audio generation //
  //////////////////////
  
  wire [15:0] audio_left;
  wire [15:0] audio_right;
  
  // a general frequency divider 
  // to generate the required frequencies for speaker clock
  note_gen Ung(
    .audio_left,  // left sound audio
    .audio_right, // right sound audio
    .note_div(27'd100_000_000 / 2'd2 / note_frequency),
    .clk,         // clock from crystal
    .rst_n        // active low reset
  );
  
  ////////////////////////
  // Speaker controllor //
  ////////////////////////
  
  // a stereo signal parallel-to-serial processor
  // to generate the speaker control signals
  speaker_control Usc(
    // outputs
    .audio_mclk,  // master clock
    .audio_lrck,  // left-right clock
    .audio_sck,   // serial clock
    .audio_sdin,  // serial audio data input
    // inputs
    .audio_left,  // left channel audio data input
    .audio_right, // right channel audio data input
    .clk,         // clock from the crystal
    .rst_n        // active low reset
  );
  
  ///////////////////////
  // look-up note name //
  ///////////////////////
  
  reg [15:0] note_name;
  
  always @* begin
    case ('b1)
      key_down[8'h1C]: note_name <= 16'b10000101_11000101; // Do
      key_down[8'h1B]: note_name <= 16'b11110101_01100001; // Re
      key_down[8'h23]: note_name <= 16'b00101011_11011111; // Mi
      key_down[8'h2B]: note_name <= 16'b01110001_00010001; // Fa
      key_down[8'h34]: note_name <= 16'b01001001_11000101; // So
      key_down[8'h33]: note_name <= 16'b11100011_00010001; // La
      key_down[8'h3B]: note_name <= 16'b01001001_11011111; // Si
      key_down[8'h15]: note_name <= 16'b10000101_11000101; // Do
      key_down[8'h1D]: note_name <= 16'b11110101_01100001; // Re
      key_down[8'h24]: note_name <= 16'b00101011_11011111; // Mi
      key_down[8'h2D]: note_name <= 16'b01110001_00010001; // Fa
      key_down[8'h2C]: note_name <= 16'b01001001_11000101; // So
      key_down[8'h35]: note_name <= 16'b11100011_00010001; // La
      key_down[8'h3C]: note_name <= 16'b01001001_11011111; // Si
      default: note_name <= 16'hFFFF;
    endcase
  end
  
  /////////////////
  // control ssd //
  /////////////////
  
  // clock for persistence of vision
  reg [20:0] clk_f;

  always @(posedge clk or negedge rst_n) begin
    if (~rst_n) clk_f <= 'b0;
    else        clk_f <= clk_f + 'b1;
  end

  // anode enable
  always @* begin
    case (clk_f[20:19])
      2'd0:    anode = 4'b1110;
      2'd1:    anode = 4'b1101;
      2'd2:    anode = 4'b1011;
      2'd3:    anode = 4'b0111;
      default: anode = 4'b0000;
    endcase
  end
  
  // cathode select
  reg [ 3:0] now_display;
  
  always @* begin
    case (clk_f[20:19])
      2'd0:    cathode = note_name[ 7:0];
      2'd1:    cathode = note_name[15:8];
      2'd2:    cathode = 8'hFF;
      2'd3:    cathode = 8'hFF;
      default: cathode = 8'hFF;
    endcase
  end
  
endmodule
